
open Dominion

exception End_of_Game of Deck.t list

let ask player phase limit decks supply =
  match phase with
    | Phase.Action -> None
    | Phase.Treasure -> 
	begin
	  try
	    let myhand = Deck.get_hand (ListUtil.at decks player) in
	      Some 
		(List.hd 
		   (List.filter 
		      (fun x -> 
			 match cardtype_of_card x with 
			   | Cardtype.Treasure _ -> true 
			   | _ -> false) 
		      myhand))
	  with Failure "hd" -> None
	end
    | Phase.Purchase -> 
	begin
	  if limit.buy = 0 then None
	  else match limit.money with
	    | 0 | 1 | 2 -> None
	    | 3 | 4 | 5 -> Some Silver
	    | 6 | 7     -> Some Gold
	    | _         -> Some Province
	end
    | Phase.Cleanup -> assert false

let _ =
  Random.self_init ();
  let numplayer = 2 in
  let deck0 = Deck.init_deck () in
  let deck1 = Deck.init_deck () in
    let rec loop game =
      let (decks, (phase, player, limit), supply, numplayer, turn) = game in
	if phase = Phase.Cleanup then
	  if Supply.endgame supply || turn >= 100 then
	    raise (End_of_Game decks)
	  else 
	    let newdecks = 
	      ListUtil.change_to 
		decks 
		player 
		(Deck.cleanup (ListUtil.at decks player))
	    in
	      loop (newdecks,
		    ((Phase.next phase), 
		     (player + 1) mod numplayer, 
		     {action = 1; money = 0; buy = 1}), 
		    supply,
		    numplayer,
		    turn + 1)
	else
	  match ask player phase limit decks supply with
	    | None -> loop (decks, (Phase.next phase, player, limit), supply, numplayer, turn)
	    | Some card ->
		begin
		  match phase with
		    | Phase.Action ->
			if limit.action = 0 then assert false
			else
			  begin
			    match cardtype_of_card card with
			      | Cardtype.Action
			      | Cardtype.Attack
			      | Cardtype.Reaction -> assert false
			      | _ -> assert false
			  end
		    | Phase.Treasure -> 
			begin
			  match cardtype_of_card card with
			    | Cardtype.Treasure x
				-> loop ((ListUtil.change_to 
					    decks 
					    player 
					    (Deck.play (ListUtil.at decks player) card)),
					 (Phase.Treasure, 
					  player,
					  { action = limit.action; money = limit.money + x; buy = limit.buy }),
					 supply, 
					 numplayer,
					 turn)
			    | _ -> assert false
			end
		    | Phase.Purchase ->
			if limit.buy = 0 || 
			   not (Supply.exist supply card) || 
			   (cost_of_card card) > limit.money
			then assert false
			else 
			  let newsupply = Supply.decrease supply card in
			  let newdeck = Deck.obtain (ListUtil.at decks player) card in
			    loop ((ListUtil.change_to
				     decks
				     player
				     newdeck),
				  (Phase.Treasure,
				   player,
				   { action = limit.action; 
				     money = limit.money - (cost_of_card card);
				     buy = limit.buy - 1}),
				  newsupply,
				  numplayer,
				  turn)
		    | _ -> assert false
		end
    in
      try
	loop ([deck0; deck1], 
	      (Phase.Action, 
	       0, 
	       {action = 1; money = 0; buy = 1}), 
	      (Supply.create numplayer),
	      numplayer,
	      0)
      with End_of_Game decks ->
	let rec print_points decks iter =
	  match decks with
	    | [] -> ()
	    | x::xs -> 
		print_endline ("player" ^ (string_of_int iter) ^ " : " ^ (string_of_int x)); 
		print_points xs (iter+1)
	in
	  print_points (List.map Deck.count_victory decks) 0;;
